##
# PloneSVNaccess<br>
#  Providing read and write access to subversion.<br>
#<br>
# PloneSVNaccessTool.py<br>
#  Providing the PloneSVNaccessTool (important!) and SVNFileOrFolder.<br>
#<br>
# Author: Christian Koller 'admin@iecw.net'.<br>
# Extended by: Thomas Mller<br>
#<br>
# Released under GNU GPL v2 or later.
##

from Products.CMFCore.utils import UniqueObject 

from OFS.SimpleItem import SimpleItem 

from Globals import InitializeClass 
from zLOG import LOG, INFO
from AccessControl import ClassSecurityInfo
import pysvn 
import time
import os
from os.path import join
import tarfile

from re import *

import zipfile
import locale

##
# PloneSVNaccess Tool<br>
# The tool provides the access to the subversion repository
class PloneSVNaccessTool (UniqueObject, SimpleItem): 

	""" PloneSVNaccess Tool
		The tool provides the access to the subversion repository
	""" 
	

	id = 'PloneSVNaccessTool' 

	meta_type= 'Plone SVN access Tool' 

	plone_tool = 1 

	security = ClassSecurityInfo()



	security.declarePrivate("setEncoding")
	##
	# Set encoding to prevent problems with the c library pysvn<br>
	# 
	# Info: <a href="http://pysvn.tigris.org/servlets/ReadMsg?listName=users&msgNo=257"> pysvn msgNo=257 </a>
	#
	def setEncoding(self):
		""" Set encoding to prevent problems with the c library pysvn
		
			Info: http://pysvn.tigris.org/servlets/ReadMsg?listName=users&msgNo=257
		"""
		language_code, encoding = locale.getdefaultlocale() 
		#LOG('PloneSVNaccessTool', 0, 'language_code:', language_code) 
		#LOG('PloneSVNaccessTool', 0, 'encoding:', encoding) 

		if language_code is None: 
			language_code = 'de_DE' 
		encoding = 'UTF-8' 
		locale.setlocale(locale.LC_CTYPE, '%s.%s' % (language_code, encoding) ) 



	security.declarePrivate("setPloneSVNaccessVersioning")
	##
	# Set PloneSVNaccessVersioning<br>
	# (Called when the user downloads a file)
	#
	def setPloneSVNaccessVersioning(self, svn_path, fileOrfolder, revNo):
		""" Set PloneSVNaccessVersioning
			(Called when the user downloads a file)
		"""
		#LOG('PloneSVNaccessTool', 0, 'setPloneSVNaccessVersioning:', revNo)
		revNo = int(revNo)
		
		user = self.portal_membership.getAuthenticatedMember()
		try:
			ploneSVNaccessVersioning = user.getProperty("PloneSVNaccessVersioning")
			try:
				ploneSVNaccessVersioning[svn_path][fileOrfolder] = revNo
			except KeyError:
				#IF no PloneSVNaccessVersioning - fileOrfolder entry
				ploneSVNaccessVersioning[svn_path] = {fileOrfolder: revNo}
			except TypeError:
				pass
				#Seem to be anonymous
				
			user.setMemberProperties({"PloneSVNaccessVersioning": ploneSVNaccessVersioning})			
		except ValueError:
			# IF no PloneSVNaccessVersioning entry
			user.setMemberProperties({"PloneSVNaccessVersioning": {svn_path: {fileOrfolder: revNo} } })
		except AttributeError:
			pass
			#Seem to be anonymous



	security.declarePublic("getPloneSVNaccessVersioning")
	##
	# Get PloneSVNaccessVersioning<br>
	# (False or the Versioning number of the last download through this user)
	#
	def getPloneSVNaccessVersioning(self, svn_path, fileOrfolder):		
		""" Get PloneSVNaccessVersioning
			(False or the Versioning number of the last download through this user)
		"""
		try:
			user = self.portal_membership.getAuthenticatedMember()
			ploneSVNaccessVersioning = user.getProperty("PloneSVNaccessVersioning")
			ploneSVNaccessVersioningNo = ploneSVNaccessVersioning[svn_path][fileOrfolder]
			return ploneSVNaccessVersioningNo
		except:
			return False



	security.declarePublic("createCommOwWaJS")
	##
	# Create the Java-Script for the commit action.<br>
	# (Called when the file is binary and the file was changed since the last download)
	#
	def createCommOwWaJS(self, curSVNFile, curFileLog):
		""" Create commit overewrite warning JS
			Create the Java-Script for the commit action.
			(Called when the file is binary and the file was changed since the last download)
		"""
		
		changers = ""
		for line in curFileLog:
			changers = changers + line.author + ", "
			
		return 	u"""
					return confirm('The file "%s" has been changed through "%s" since your last download. You will overwrite this changes!');
				""" % ( curSVNFile, changers[:-2] )




	security.declarePublic("createDelJSCheck")
	##
	# Create the Java-Script for the delete action
	#
	def createDelJSCheck(self, curSVNFolder, curSVNfile, username):
		""" Create Delete JS Check
			Create the Java-Script for the delete action
		"""
		return 	u"""
					check = prompt('Delete: "%s/%s" ?      Enter the log message:', '%s');
					if (check == undefined) { return false; }
					if (check == 'stop') { return false; }
					window.location.href = this + '&curMessage=' + check;
					return false;
				""" % ( curSVNFolder, curSVNfile, self.getLogMessage(username) )




	##
	# Delete File or Folder within SVN
	#
	def deleteFileOrFolder(self, svn_path, username, password, fileOrfolder, curMessage):
		""" Delete File or Folder within SVN
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		if not curMessage:
			self.log_message = "'" + fileOrfolder + "' deletet at " + time.strftime(' - %d.%m.%Y %H:%M:%S: ', time.localtime())
		else:
			self.log_message = curMessage
		
		# Get it removed"

		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login			

			client.callback_get_log_message = self._get_log_message

			client.remove( (svn_path.encode("utf-8") + fileOrfolder).decode("utf-8") )
			
			return "Folder '" + fileOrfolder + "' deleted."
			
		except pysvn.ClientError, e:
			return "Error: " + str(e)



	security.declarePrivate("_del_Temp_Folder")
	##
	# Delete local temp folder<br>
	# Delete everything reachable from the directory named in 'tempDir'.<br>

	# CAUTION:  This is dangerous!  For example, if tempDir == '/', it

	# could delete all your disk files.<br>
	# From: <a href="http://python.org/doc/2.3.5/lib/os-file-dir.html"> Python Doc </a>

	#
	def _del_Temp_Folder(self, tempDir):
		""" Delete !!! Local !!! Temp Folder
			Delete everything reachable from the directory named in 'tempDir'.

			CAUTION:  This is dangerous!  For example, if tempDir == '/', it could delete all your disk files.

			From: http://python.org/doc/2.3.5/lib/os-file-dir.html
		"""
		# Remove temp folder
		for root, dirs, files in os.walk(tempDir, topdown=False):

			for name in files:

				os.remove(join(root, name))

			for name in dirs:

				os.rmdir(join(root, name))
		os.rmdir(tempDir)



	##
	# Commit a file to SVN<br>
	# May raise some SVNConnectError<br>
	# @return If the reply starts with 'SVNMERGE' the file needs to be merged. The text behind SVNMERGE ist
	# the result of SVN update.<br>
	# Without SVNMERGE you get a status message.<br>
	#
	def commit_svnFile(self, svn_path, username, password, curSVNFolder, curSVNFile, curMessage, curFile=None, curFileContent=None):
		""" Commit a file to SVN
			May raise some SVNConnectError
			If the reply starts with 'SVNMERGE' the file needs to be merged. The text behind SVNMERGE ist
			the result of SVN update.
			Without SVNMERGE you get a status message.
		"""
		
		if not curFile and not curFileContent:
			raise SVNConnectError("Error: We need either curFile or curFileContent.")
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		filenameStr = (curSVNFile.split("/").pop())
		warnNoVersioning = False

		# The temp folder
		tempDir = os.tmpnam()
		os.mkdir(tempDir)
		# The temp file
		tempFileName = tempDir.encode("utf-8") + os.sep.encode("utf-8") + filenameStr

		# Try to add it to the SVN		
		try:
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
			client.exception_style = 1
	
			fileRevNo = self.getPloneSVNaccessVersioning(svn_path, curSVNFile)
			if fileRevNo and curFile:
				# Checkout PloneSVNaccessVersioning with RevNo = fileRevNo if we have curFile
				client.checkout( (svn_path.encode("utf-8") + curSVNFolder).decode("utf-8") , tempDir, recurse=False, revision=pysvn.Revision( pysvn.opt_revision_kind.number, int(fileRevNo)))
			else:
				# Checkout HEAD if we have curFileContent
				warnNoVersioning = True
				client.checkout( (svn_path.encode("utf-8") + curSVNFolder).decode("utf-8") , tempDir, recurse=False)
	
			# Copy file to temp file
			tempFile = file(tempFileName, "w+b")
			if curFile:
				# Copy the curFile to tempFile
				while True:
					theFileContentLine = curFile.read()
					if not theFileContentLine:
						break
					tempFile.write(theFileContentLine)
				curFile.close()
			else:
				# Copy the curFileContent to tempFile
				tempFile.write(curFileContent)
			tempFile.close()
						
			client.update(tempDir)
			
			changes = client.status(tempFileName.decode("utf-8"))
			if changes[0].text_status == pysvn.wc_status_kind.normal:
				return "No commit required: File '" + filenameStr + "' not modified"
				# Nothing to do
				
			elif changes[0].text_status == pysvn.wc_status_kind.modified:
				# Commit file
				client.checkin(tempFileName.decode("utf-8"), curMessage.decode("utf-8"))
				
				# Set PloneSVNaccessVersioning
				aktRevNo = self.getMaxRevNo( (svn_path.encode("utf-8") + curSVNFile).decode("utf-8") , username, password)
				self.setPloneSVNaccessVersioning(svn_path, curSVNFile, aktRevNo)
								
				# Delete Temp Dir
				self._del_Temp_Folder(tempDir)
		
				return "File '" + filenameStr + "' commit ok."
				# Commit was ok "
				
			elif changes[0].text_status == pysvn.wc_status_kind.conflicted and curFile:
				# Merge file content
				# Get the file content
				theFileFile = file(tempFileName, "r")
				theFileContent = ""
				while True:
					theFileContentLine = theFileFile.read()
					if not theFileContentLine:
						break
					theFileContent = theFileContent + theFileContentLine
				theFileFile.close()
				
				# Delete Temp Dir
				self._del_Temp_Folder(tempDir)
				
				# Set the file Content
				return "SVNMERGE" + theFileContent
				
			else:
				# Delete Temp Dir
				self._del_Temp_Folder(tempDir)
	
				return "File '" + filenameStr + "' " + str(changes[0].text_status)
				# Nothing to do?"
				
		except pysvn.ClientError, e:
			self._del_Temp_Folder(tempDir)
			return "Error: " + str(e)


	##
	# Upload (import) a file to SVN
	#
	def upload_svnFile(self, username, password, theFullPath, curFile, curMessage):
		""" Upload (import) a file to SVN
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		filename = curFile.filename
		
		self.setEncoding()

		# The temp folder'
		tempDir = os.tmpnam()
		os.mkdir(tempDir)
		# The temp file
		tempFileName = tempDir.encode("utf-8") + os.sep.encode("utf-8") + filename

		#LOG('PloneSVNaccessTool', 0, 'tempFileName:', tempFileName)

		tempFile = file(tempFileName, "w+b")

		# Copy file to temp file
		while True:
			theFileContent = curFile.read()
			if not theFileContent:
				break
			tempFile.write(theFileContent)
		curFile.close()
		tempFile.close()
		
		# Try to add it to the SVN
		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
									
			client.import_(tempFileName.decode("utf-8"), theFullPath.decode("utf-8") , curMessage.decode("utf-8"))
			
			self._del_Temp_Folder(tempDir)
	
			return "File '" + filename + "' upload ok."
			
			
		except pysvn.ClientError, e:
			self._del_Temp_Folder(tempDir)
			return "Error: " + str(e)
		
		

	security.declarePublic("getLogMessage")
	##
	# Get Message for log creates a simple log message
	#
	def getLogMessage(self, username):
		""" Get Message for log creates a simple log message
		"""
		return username() + time.strftime(' - %d.%m.%Y %H:%M:%S: ', time.localtime())



	##
	# _get_login will be called when we need authentication
	#
	def _get_login(self, realm, username, may_save):
		""" _get_login will be called when we need authentication
		"""
		
		# Info
		#LOG('PloneSVNaccessTool', 0, 'Get login:', 'realm:' + realm + ' - username:' + username + ' - self.username:' + self.username + ' - self.password:' + self.password)
		
		if hasattr(self, "username"):
			self.loginCount += 1
			if self.loginCount > 10:
				#LOG('PloneSVNaccessTool', 0, 'Get login:', 'Invalid username/password for realm: ' + realm)
				return False, '', '', False
		
		# Check, if not empty
		if self.username == '':
			return False, '', '', False
		else:
			return True, self.username, self.password, False



	##
	# _get_log_message will be called when we need a log message
	#
	def _get_log_message(self):
		""" _get_log_message will be called when we need a log message
		"""
		return True, self.log_message



	##
	# Get the folder listing
	#
	def getDir(self, svn_path, username, password, folder, revNo):
		""" Get the folder listing
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		revNo = int(revNo)
		
		# Info
		#LOG('PloneSVNaccessTool', 0, 'getDir:', 'svn_path:' + svn_path + ' - username:' + username + ' - password:' + password + ' - folder:' + folder + ' revNo:' + str(revNo))

		try:
			pathToDirFile = svn_path + folder
		except:
			pathToDirFile = (svn_path.encode("utf-8") + folder).decode("utf-8")

		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
			
			entry_list = client.ls( pathToDirFile , pysvn.Revision(pysvn.opt_revision_kind.number, revNo))
			
			return [SVNFileOrFolder(self.mimetypes_registry, ** dict(single_entry) ) for single_entry in entry_list]
			
		except pysvn.ClientError:
			return False


	##
	# Get the file info
	#
	def getFile(self, svn_path, username, password, curSVNFile, revNo):
		""" Get the file info
		"""
		
		try:	
			return self.getDir(svn_path, username, password, curSVNFile, revNo)[0]
		except:
			return False



	##
	# Return the RevNo of head
	#
	def getMaxRevNo(self, svn_path, username, password):
		""" Return the RevNo of head
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
				
		# Info
		#LOG('PloneSVNaccessTool', 0, 'returnMaxRevNo:', 'svn_path:' + svn_path + ' - username:' + username + ' - password:' + password)

		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
			
			return client.log(svn_path)[0]['revision'].number
	
		except pysvn.ClientError:
			return ''



	##
	# Return Log info
	#
	def getLog(self, svn_path, username, password, folder, curRevNo, startRevNo = None, trackerurl = '', revlinks = True):
		""" Return Log info
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		curRevNo = int(curRevNo)
		if startRevNo:
			startRevNo = int(startRevNo)
		else:
			startRevNo = 0
		
		# Info
		#LOG('PloneSVNaccessTool', 0, 'getLog:', 'svn_path:' + svn_path + ' - username:' + username + ' - password:' + password + ' - folder:' + folder + ' - curRevNo:' + str(curRevNo) + ' - startRevNo:' + str(startRevNo) + ' - trackerurl:' + str(trackerurl))

		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
						
			entry_list = client.log( svn_path.encode("utf-8") + folder ,
				revision_start = pysvn.Revision( pysvn.opt_revision_kind.number, curRevNo ),
				revision_end = pysvn.Revision( pysvn.opt_revision_kind.number, startRevNo ),
				strict_node_history=False,)
			
			# Extended by Thomas Mller:
			svnlog_list = [SVNLogInfo(** dict(single_entry)) for single_entry in entry_list]

			if revlinks:
				pattern = compile('r[0-9]+')
				for log_entry in svnlog_list:
					pos = 0
					while True:
						res = pattern.search(log_entry.message, pos)
						if res == None:
							break;
						pos = res.start() + 1
						rev = res.string[pos: res.end()]
			
						if rev.isdigit():
							link = '<a href="?curRevNo=' + rev + '">r'+rev+'</a>'
							log_entry.message = log_entry.message[0: pos-1] + link + log_entry.message[res.end():]
							pos += len(link)
				        #LOG('PloneSVNaccessTool', 0, log_entry.message)

			if len(trackerurl) == 0:
				#LOG('PloneSVNaccessTool', 0, 'No trackerurl set. Bug link creation disabled.')
				return svnlog_list

			pattern = compile('#[0-9]+')
			for log_entry in svnlog_list:
				pos = 0
				while True:
					res = pattern.search(log_entry.message, pos)
					if res == None:
						break;
					pos = res.start() + 1
					bug = res.string[pos: res.end()]
				
					if bug.isdigit():
						link = '<a href="' + trackerurl %{'bug' : bug} + '">#'+bug+'</a>'
						log_entry.message = log_entry.message[0: pos-1] + link + log_entry.message[res.end():]
						pos += len(link)
			        #LOG('PloneSVNaccessTool', 0, log_entry.message)

			return svnlog_list
			
		except pysvn.ClientError:
			return False	



	##
	# Return file Diff
	#
	def getDiff(self, svn_path, username, password, curSVNFile, curRevNo, otherRevNo):
		""" Return file Diff
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		curRevNo = int(curRevNo)
		revision1 = pysvn.Revision(pysvn.opt_revision_kind.number, curRevNo)
		otherRevNo = int(otherRevNo)
		revision2 = pysvn.Revision(pysvn.opt_revision_kind.number, otherRevNo)
		pathToFile = svn_path.encode("utf-8") + curSVNFile
				
		# The temp folder
		tempDir = os.tmpnam()

		# Info
		#LOG('PloneSVNaccessTool', 0, 'getDiff:', 'svn_path:' + svn_path + ' - username:' + username + ' - password:' + password + ' - curSVNFile:' + curSVNFile + ' - curRevNo:' + str(curRevNo) + " - otherRevNo:" + str(otherRevNo) + " - tempDir:" + tempDir)

		#os.mkdir(tempDir)
		
		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
						
			return client.diff(tempDir, url_or_path = pathToFile, revision1 = revision1, revision2 = revision2)
			
		except:
			return False



	security.declarePublic("getNavigationParent")
	##
	# Return the link to the parent dir
	#
	def getNavigationParent(self, path):
		""" Return the link to the parent dir
		"""
		
		ret = path.split("/")
		ret.pop()
		return "/".join(ret)



	##
	# Returns the name of all svn breadcrumbs
	#
	def getBreadCrumbs(self, path):
		""" Returns the name of all svn breadcrumbs
		"""
		
		split_path = path.split("/")
		ret = []
		all_subs = []
		for path_part in split_path:
			if path_part != '':
				all_subs.append(path_part)
				ret.append([path_part,"/".join(all_subs)])
		return ret



	##
	# Returns revision numbers of the folder or file<br>
	# Limited to 15 values
	#
	def getRevNos(self, curRevNo, maxRevNo, svn_path, username, password, folderOrFile):
		""" Returns revision numbers of the folder or file
			Limited to 15 values
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		curRevNo = int(curRevNo)
		maxRevNo = int(maxRevNo)

		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
			
			returnList = []
			# Current Rev No + 1 to maxRevNo
			for aktRevNo in range(curRevNo+1, maxRevNo+1):
				try:
					entry_list = client.log( svn_path + folderOrFile ,
						revision_start = pysvn.Revision( pysvn.opt_revision_kind.number, aktRevNo ),
						revision_end = pysvn.Revision( pysvn.opt_revision_kind.number, aktRevNo ),
						strict_node_history=False,)
					returnList.append([aktRevNo,time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(entry_list[0]['date']))])
					# Break when enough
					if len(returnList) >= 7:
						break
				except:
					pass

			# Current Rev No to 1
			for aktRevNo in range(curRevNo, 0, -1):
				try:
					entry_list = client.log( svn_path + folderOrFile ,
						revision_start = pysvn.Revision( pysvn.opt_revision_kind.number, aktRevNo ),
						revision_end = pysvn.Revision( pysvn.opt_revision_kind.number, aktRevNo ),
						strict_node_history=False,)
					returnList.append([aktRevNo,time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(entry_list[0]['date']))])
					# Break when enough
					if len(returnList) >= 15:
						break
				except:
					pass

			returnList.sort()
			return returnList			
		except:
			return False			



	##
	# Get a single file<br>
	#
	# @keyparam errorType 0 -> raise all errors<br>
	# 1 -> reply with False if file is not available with this revision number<br>
	# @keyparam forDownload True -> set PloneSVNaccessVersioning
	#
	def getSingleFile(self, svn_path, username, password, curSVNFile, revNo, errorType, forDownload):
		"""	Get a single file

			errorType=0 raise all errors
			errorType=1 reply with False if file is not available with this revision number
			forDownload = True -> set PloneSVNaccessVersioning
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		revNo = int(revNo)
		revision = pysvn.Revision(pysvn.opt_revision_kind.number, revNo)
		try:
			pathToFile = svn_path + curSVNFile
		except:
			pathToFile = (svn_path.encode("utf-8") + curSVNFile).decode("utf-8")
		
		try:		
			self.loginCount = 0
			self.setEncoding()
			client = pysvn.Client()
			client.callback_get_login = self._get_login
			
			infoAboutFile = self.getFile(svn_path, username, password, curSVNFile, revNo)

			theFileContent = client.cat(pathToFile, revision)
			
			# Try to find a valid mime-type
			try:
				theMimeTypeInfo = client.revproplist(svn_path)[0][1].get("svn:mime-type", None)
				if theMimeTypeInfo == "application/octet-stream":
					theMimeTypeInfo = None	# In http://www.rfc-editor.org/rfc/rfc2045.txt page 17: 
										# Any entity with an unrecognized Content-Transfer-Encoding must be

										# treated as if it has a Content-Type of "application/octet-stream"
			except:
				theMimeTypeInfo = None
			# Try to find a mime-type in the registry
			if not theMimeTypeInfo and self.mimetypes_registry:
				theMimeTypeInfo = self.mimetypes_registry.lookupExtension(curSVNFile.split("/").pop())
			if not theMimeTypeInfo:
				theMimeTypeInfo = "application/octet-stream" # Nothing found...
				mimeStr = "Unknown"
			else:
				mimeStr = theMimeTypeInfo.name
				theMimeTypeInfo = theMimeTypeInfo.normalized()
				
			# Set PloneSVNaccessVersioning
			if forDownload:
				self.setPloneSVNaccessVersioning(svn_path, curSVNFile, revNo)
			
			return SVNFile(theFileContent, theMimeTypeInfo, infoAboutFile, mimeStr)
							
		except pysvn.ClientError:			
			message = "Can not connect to svn server! User solution: Call the administrator. Administrator solution: Check the username/password. Check if the server is up and running. Check if the file is available!" 
			raise SVNConnectError(message)



	##
	# Get a folder as tar or zip<br>
	# checkout and export always uses the maxRevNo = HEAD
	# @keyparam fileFormat 'tar' -> .tar.gz file<br>
	# 'zip' -> .zip file<br>
	# @keyparam exportType 'export' -> SVN export<br>
	# 'checkout' -> SVN checkout
	#
	def getFolderAsTarOrZip(self, svn_path, username, password, curSVNFolder, fileFormat, exportType, maxRevNo):
		"""	Get a folder as tar or zip
			checkout and export always uses the maxRevNo = HEAD
			fileFormat:
			  'tar' -> .tar.gz file<br>
			  'zip' -> .zip file<br>
			exportType:
			  'export' -> SVN export<br>
			  'checkout' -> SVN checkout
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		maxRevNo = int(maxRevNo)
		pathToFile = svn_path.encode("utf-8") + curSVNFolder
		
		# The temp folder
		tempDir = os.tmpnam()
		# The temp file
		tempFile = os.tmpfile()
		
		self.loginCount = 0
		self.setEncoding()
		client = pysvn.Client()
		client.callback_get_login = self._get_login

		# Get the folder content
		if exportType == "export":		
			client.export(pathToFile, tempDir)
		else:
			client.checkout(pathToFile, tempDir)
		
		if fileFormat == "tar":
			# Create the tar file
			theTarFile = tarfile.open(fileobj=tempFile, mode='w:gz', name="tempfile.tar.gz")
		else:
			# Create the zip file		
			theZipFile = zipfile.ZipFile(tempFile, 'w')

		# Add files
		for root, dirs, files in os.walk(tempDir, topdown=False):
			for name in files:
				nameLatin = join(root.replace(tempDir,''), name)
				if fileFormat == "tar":
					theTarFile.add(join(root, name), nameLatin)				
				else:
					theZipFile.write(join(root, name), nameLatin)
				# Set PloneSVNaccessVersioning
				if exportType != "export":
					# Write versioning only for checkout
					if nameLatin[:1] != '/':
						nameLatin = '/' + nameLatin
					if nameLatin.find('/.svn') == -1:
						self.setPloneSVNaccessVersioning(svn_path, curSVNFolder + nameLatin, maxRevNo)
			#Ignore dirs - Do not log them within the PloneSVNaccessVersioning

		# Close files		
		if fileFormat == "tar":
			theTarFile.close()
		else:
			theZipFile.close()
		
		self._del_Temp_Folder(tempDir)
		
		return tempFile



	##
	#  Create Folder in the subversion repository
	#
	def createFolder(self, username, password, theFolderName, curMessage, theFullPath):
		""" Create Folder in the subversion repository
		"""
		
		self.username = username.encode("utf-8")
		self.password = password.encode("utf-8")
		if not curMessage:
			self.log_message = ( " Folder '" + theFolderName + "' created at " + time.strftime(' - %d.%m.%Y %H:%M:%S: ', time.localtime()) ).decode("utf-8")
		else:
			self.log_message = curMessage.decode("utf-8")
		
		# Info "
		#LOG('PloneSVNaccessTool', 0, 'createFolder', theFullPath)

#		try:		
		self.loginCount = 0
		self.setEncoding()
		client = pysvn.Client()
		client.callback_get_login = self._get_login			

		client.mkdir(theFullPath.decode("utf-8"), self.log_message)
		
		return "Folder '" + theFolderName + "' created."
			
#		except pysvn.ClientError, e:
#			return "Error: " + str(e)

# Initialize the PloneSVNaccessTool class
InitializeClass(PloneSVNaccessTool) 



##
# Represents information about a file or folder
#
class SVNFileOrFolder:
	def __init__(self, mimetypes_registry, **lsreply):
		""" Represents information about a file or folder
		"""
		
		#Info
		#LOG('PloneSVNaccessTool', 0, 'lsreply:', lsreply)
		
		self.isDir = lsreply['kind'] == pysvn.node_kind.dir
		self.name = lsreply['name'].split("/").pop()
		self.revNo = lsreply['created_rev'].number
		self.revNoStr = str(lsreply['created_rev'].number)
		self.date = lsreply['time']
		self.dateStr = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(lsreply['time']))
		self.size = lsreply['size']
		self.author  = lsreply['last_author']
		
		if not self.isDir:
			theMimeTypeInfo = mimetypes_registry.lookupExtension(self.name.split("/").pop())
			if not theMimeTypeInfo:
				self.mime = "application/octet-stream" # Nothing found...
				self.mimeStr = "Unknown"
			else:
				self.mimeStr = theMimeTypeInfo.name
				self.mime = theMimeTypeInfo.normalized()

			self.isAudio = self.mime.find("audio/") == 0
			self.isText = self.mime.find("text/") == 0
			self.isImage = self.mime.find("image/") == 0
			self.isVideo = self.mime.find("video/") == 0
			self.isPDF = self.mime.find("application/pdf") == 0
			self.isZip = self.mime.find("tar") > 0 or self.mime.find("rar") > 0 or self.mime.find("zip") > 0 or self.mime.find("cpio") > 0
			self.isExe = self.mime.find("application/octet-stream") == 0 or self.mime.find("executable") > 0
			
			self.isOther = not self.isText and not self.isImage and not self.isPDF and not self.isAudio and not self.isZip and not self.isExe and not self.isVideo
				
	security = ClassSecurityInfo()
	security.declareObjectPublic()
	security.setDefaultAccess("allow")
	
	def get_size(self):
		return self.size

# Initialize the SVNFileOrFolder class
InitializeClass(SVNFileOrFolder) 



##
# Represents a single file incl. file content
#
class SVNFile:
	def __init__(self, file_content, mime, info, mimeStr):
		""" Represents a single file incl. file content
		"""
				
		self.file_content = file_content
		self.mime = mime
		self.info = info
		self.mimeStr = mimeStr		
		self.isText = mime.find("text/") == 0
		self.isImage = mime.find("image/") == 0

	security  = ClassSecurityInfo()
	security.declareObjectPublic()
	security.setDefaultAccess("allow")
	
# Initialize the SVNFile class
InitializeClass(SVNFile) 



##
# A Simple SVNConnectError Error
#
class SVNConnectError(Exception):

	def __init__(self, value):

		""" A Simple SVNConnectError Error
		"""
		self.value = value
		

	def __str__(self):

		""" Return the string value of the SVNConnectError Error
		"""
		return repr(self.value)

# Initialize the SVNFile class
InitializeClass(SVNConnectError)



##
# A Log Info
#
class SVNLogInfo:

	def __init__(self, **logEntry):
		""" A Log Info
		"""
		
		self.author = logEntry['author']
		self.date = logEntry['date']
		self.dateStr = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(logEntry['date']))
		self.message = logEntry['message']
		self.revNo = logEntry['revision'].number
		self.revNoStr = str(logEntry['revision'].number)
		self.revDate =	str(logEntry['revision'].date) #n.u.
		self.kind = logEntry['revision'].kind #n.u.
		self.kindStr = str(logEntry['revision'].kind) #n.u.

	security  = ClassSecurityInfo()
	security.declareObjectPublic()
	security.setDefaultAccess("allow")

# Initialize the SVNFile class
InitializeClass(SVNLogInfo) 


